#ifndef CFENTRYDIAMETER_H
#define CFENTRYDIAMETER_H

#include <cmath>

#include "../base/attributecalculator.h"

namespace CluE
{
/**
 * @brief Calculates the diameter D of a given CFEntry.
 * @sa T. Zhang, R. Ramakrishan and M. Livny. "BIRCH: A New Data Clustering Algorithm and Its Applications", p. 146. <em>Data Mining and Knowledge Discovery</em>, 10.1023/A:1009783824328, 1997.
 */
template<typename T> class CFDiameter : public AttributeCalculator<CFEntry<T> >
{
public:
	virtual CFDiameter* clone() const;
	
	virtual double calculate(CFEntry<T> const&) const;
};

template<typename T> CFDiameter<T>* CFDiameter<T>::clone() const
{
	return new CFDiameter(*this);
}

template<typename T> double CFDiameter<T>::calculate(CFEntry<T> const & x) const
{
	double numerator = 2*(x.number*x.SS - x.LS*x.LS);
	double denominator = x.number * (x.number-1);
	return std::pow(numerator / denominator, 0.5);
}

}

#endif
