#ifndef DISTINCTELEMENTS_H
#define DISTINCTELEMENTS_H

#include <set>
#include <functional>

#include "../hashing/hashfunction.h"

namespace CluE
{

/**
 * @brief Count distinct elements in a data stream
 *
 * @warning This implementation is still considered to be unstable.
 */
template<typename U, typename H> class DistinctElements
{

public:
	DistinctElements(unsigned long long sizeOfUniverse, double eps, std::function<HashFunction<U,H>*(unsigned long long sizeOfHashingSpace)> hashfunctionCreator) :
		sizeOfUniverse(sizeOfUniverse),
		sizeOfUniverse3(sizeOfUniverse*sizeOfUniverse*sizeOfUniverse),
		rank(ceil(96/(eps*eps))),
		eps(eps),
		hashfunction(hashfunctionCreator(sizeOfUniverse3)),
		elements()
	{
	}
	
	~DistinctElements()
	{
		delete hashfunction;
	}

	DistinctElements<U,H>& operator<<(U const & element);

	unsigned long long numberOfDistinctElements();

private:
	unsigned long long sizeOfUniverse;
	unsigned long long sizeOfUniverse3;
	unsigned long long rank;
	double eps;
	HashFunction<U,H> * hashfunction;
	std::set<U> elements;
};

template<typename U, typename H> DistinctElements<U,H>& DistinctElements<U,H>::operator<<(U const & element)
{
	H hashval = (*hashfunction)(element);
	if(elements.count(hashval) == 0)
	{
		if(elements.size() < rank)
			elements.insert(hashval);
		else if(*(--elements.cend()) > hashval)
		{
			elements.erase(--elements.end());
			elements.insert(hashval);
		}
	}
	return *this;
}

template<typename U, typename H> unsigned long long DistinctElements<U,H>::numberOfDistinctElements()
{	
	if(elements.size() == 0)
		return 0;
	else
		return (sizeOfUniverse3 * rank) / *(--elements.cend());
}

}

#endif
