#ifndef DISCRETEDOUBLESOLUTION_H
#define DISCRETEDOUBLESOLUTION_H

#include "../base/solutionprovider.h"
#include "../base/discreteproxyprovider.h"
#include "../base/partitionprovider.h"

#include <vector>

//TODO Derived documentation bug

namespace CluE
{

/**
 * @brief Data structure for partitions and discrete proxies
 *
 * This struct is for use in algorithms computing partitions and discrete proxies.
 * 
 * @ingroup data_structures
 */
template<typename T> struct DiscreteDoubleSolution : public SolutionProvider,
	public DiscreteProxyProvider<T>, public PartitionProvider<T>
{
public:
	DiscreteDoubleSolution();

	virtual ~DiscreteDoubleSolution()
	{
	}

	virtual double computationtime() const;
	virtual unsigned int number_of_solutions() const;
	virtual unsigned int size_of_solution(unsigned int) const;
	
	virtual T* discrete_proxy(unsigned int n, unsigned int c) const;
	virtual std::vector<T*> discrete_proxies(unsigned int n) const;

	virtual unsigned int clustersize(unsigned int, unsigned int) const;
	virtual T* element(unsigned int, unsigned int, unsigned int) const;
	virtual std::vector<T*> cluster(unsigned int, unsigned int) const;
	virtual std::vector<std::vector<T*> > clustering(unsigned int) const;

	double seconds;
	std::vector<std::vector<T*> > proxysets;
	std::vector<std::vector<std::vector<T*> > > partitions;
};

template<typename T> DiscreteDoubleSolution<T>::DiscreteDoubleSolution() : seconds()
{
}

template<typename T> double DiscreteDoubleSolution<T>::computationtime() const
{
	return seconds;
}

template<typename T> unsigned int DiscreteDoubleSolution<T>::number_of_solutions() const
{
	unsigned int proxynum = this->proxysets.size();
	unsigned int partitionnum = this->partitions.size();
	return proxynum<partitionnum?proxynum:partitionnum;
}

template<typename T> unsigned int DiscreteDoubleSolution<T>::size_of_solution(unsigned int i) const
{
	if (i<this->number_of_solutions())
	{
		unsigned int proxysize = this->proxysets[i].size();
		unsigned int partitionsize = this->partitions[i].size();
		return proxysize<partitionsize?proxysize:partitionsize;
	}
	return 0;
}

template<typename T> T* DiscreteDoubleSolution<T>::discrete_proxy(unsigned int n, unsigned int c) const
{
	if (n<this->proxysets.size())
		if (c<this->proxysets[n].size())
				return this->proxysets[n][c];
	return NULL;
}

template<typename T> std::vector<T*> DiscreteDoubleSolution<T>::discrete_proxies(unsigned int n) const
{
	if (n<this->proxysets.size())
			return this->proxysets[n];
	return std::vector<T*>();
}

template<typename T> unsigned int DiscreteDoubleSolution<T>::clustersize(unsigned int g, unsigned int c) const
{
	if (g<this->partitions.size())
		if (c<this->partitions[g].size())
			return this->partitions[g][c].size();
	return 0;
}

template<typename T> T* DiscreteDoubleSolution<T>::element(unsigned int g, unsigned int c, unsigned int n) const
{
	if (g<this->partitions.size())
		if (c<this->partitions[g].size())
			if (n<this->partitions[g][c].size())
				return this->partitions[g][c][n];
	return NULL;
}

template<typename T> std::vector<T*> DiscreteDoubleSolution<T>::cluster(unsigned int g, unsigned int c) const
{
	if (g<this->partitions.size())
		if (c<this->partitions[g].size())
				return this->partitions[g][c];
	return std::vector<T*>();
}

template<typename T> std::vector<std::vector<T*> > DiscreteDoubleSolution<T>::clustering(unsigned int g) const
{
	if (g<this->partitions.size())
			return this->partitions[g];
	return std::vector<std::vector<T*> >();
}

}

#endif
