#ifndef PARTITIONSOLUTION_H
#define PARTITIONSOLUTION_H

#include "../base/solutionprovider.h"
#include "../base/partitionprovider.h"

#include <vector>

namespace CluE
{

/**
 * @brief Data structure for partitions.
 *
 * This struct is for use in algorithms computing partitions.
 * 
 * @ingroup data_structures
 */
template<typename T> struct PartitionSolution : public SolutionProvider,
	public PartitionProvider<T>
{
public:

	PartitionSolution();

	virtual ~PartitionSolution()
	{
	}

	virtual double computationtime() const;
	virtual unsigned int number_of_solutions() const;
	virtual unsigned int size_of_solution(unsigned int) const;
	
	virtual unsigned int clustersize(unsigned int, unsigned int) const;
	virtual T* element(unsigned int, unsigned int, unsigned int) const;
	virtual std::vector<T*> cluster(unsigned int, unsigned int) const;
	virtual std::vector<std::vector<T*> > clustering(unsigned int) const;

	double seconds;
	std::vector<std::vector<std::vector<T*> > > partitions;
};

template<typename T> PartitionSolution<T>::PartitionSolution() : seconds()
{
}

template<typename T> double PartitionSolution<T>::computationtime() const
{
	return seconds;
}

template<typename T> unsigned int PartitionSolution<T>::number_of_solutions() const
{
	return this->partitions.size();
}

template<typename T> unsigned int PartitionSolution<T>::size_of_solution(unsigned int i) const
{
	if (i<this->partitions.size())
		return this->partitions[i].size();
	return 0;
}

template<typename T> unsigned int PartitionSolution<T>::clustersize(unsigned int g, unsigned int c) const
{
	if (g<this->partitions.size())
		if (c<this->partitions[g].size())
			return this->partitions[g][c].size();
	return 0;
}

template<typename T> T* PartitionSolution<T>::element(unsigned int g, unsigned int c, unsigned int n) const
{
	if (g<this->partitions.size())
		if (c<this->partitions[g].size())
			if (n<this->partitions[g][c].size())
				return this->partitions[g][c][n];
	return NULL;
}

template<typename T> std::vector<T*> PartitionSolution<T>::cluster(unsigned int g, unsigned int c) const
{
	if (g<this->partitions.size())
		if (c<this->partitions[g].size())
				return this->partitions[g][c];
	return std::vector<T*>();
}

template<typename T> std::vector<std::vector<T*> > PartitionSolution<T>::clustering(unsigned int g) const
{
	if (g<this->partitions.size())
			return this->partitions[g];
	return std::vector<std::vector<T*> >();
}

}

#endif
