#ifndef CLUEUNIGAUSS_H
#define CLUEUNIGAUSS_H

#include "../distribution/distribution.h"

#include <random>

#ifndef M_PI
#define M_PI           3.14159265358979323846
#endif

namespace CluE
{

/**
 * @brief Univariate normal distribution.
 *
 * @ingroup distribution_classes
 */
class UniGauss : public Distribution<double>
{
public:

	UniGauss(double m, double v);
	
	/**
	 * @brief Evaluates the density of the univariate normal distribution at the given point x.
     */
	virtual double density(double const& x) const;
	
	template<typename RndEngine> double draw(RndEngine&) const;

private:
	double mean, variance;
};

template<typename RndEngine>
double UniGauss::draw(RndEngine& re) const
{
	std::normal_distribution<> dis(this->mean, this->variance);
	return dis(re);
}

}

#endif
