/**
 * @defgroup clustering_algorithms Clustering algorithms
 * @defgroup coreset_algorithms Coreset algorithms
 * @defgroup evaluation_algorithms Evaluation algorithms
 * @defgroup sampling_algorithms Sampling algorithms
 * @defgroup helper_classes Helper classes
 * @defgroup wrapper_classes Wrapper classes
 * @defgroup data_structures Data structures
 * @defgroup base_classes Base classes
 * @defgroup pointrelated_classes Point related classes
 * @defgroup exceptions CluE exceptions
 * 
 * @mainpage
 *
 * CluE is a clustering library written in C++.
 *
 * @par Please read first
 * @ref p_clue_rules
 * 
 * @page p_clue_rules Using CluE 
 *
 * @par Compiling and dependencies
 * Premake is used as build system (http://industriousone.com/premake).
 * Run <em>premake gmake</em> in the main directory to create a Makefile for GCC Make in ./build.
 * Some classes depend on the graph framework Lemon (http://lemon.cs.elte.hu/).
 *
 * @par Naming conventions
 * Suffixes used to group classes:
 * - ...Provider \n
 *   Abstract base class. Classes \em providing funcationality derive from \em Providers. Example:
 *   - ProxyProvider
 *   - SolutionProvider
 *   .
 * - ...Solution \n
 *   Data structure. Encapsulated computation \em solutions returned by algorithms. Example:
 *   - PartitionSolution
 *   - DoubleSolution
 *   .
 * .
 *
 * @par Required lifetime guarantees for paramters passed to CluE classes
 * An object passed to a non-const method (e.g. setter) as a pointer will
 * be copied if necessary, unless it is an input object
 * (e.g. points, proxies, ...) and except where otherwise stated.\n
 * Examples:
 * - <em>setMeasure(DissimilarityMeasure<T> const *measure)</em> will copy
 *   \em measure.
 * - <em>setInput(vector<T> const *proxies)</em> will propably not copy
 *   \em proxies or any \em T object.
 * - <em>setInput(vector<T*> const *input)</em> will propably not copy
 *   \em input or any \em T* pointer / \em T object.
 * .
 * In other words, when passed by a pointer
 * - objects containing logic will be copied if necessary,
 * - data structures will (propably) not.
 * .
 * This was decided on the basis that data structures might consume lots of
 * memory.
 *
 * @par Cloning objects
 * Some classes provide a \em clone() method which should be used
 * when copying objects passed by a pointer to avoid slicing effects.\n
 * Example:
 * - <em>DissimilarityMeasure<T>* clone() const</em>
 * .
 *
 * @par Unboxing objects
 * Some classes provide a static \em toType() method to unbox pointers.\n
 * Example:
 * - <em>static ProxyProvider<T>* toProxyProvider(SolutionProvider* s)</em>
 * .
 */
