#ifndef FREQUENCYDISTRIBUTION_H
#define FREQUENCYDISTRIBUTION_H

#include <vector>
#include <iostream>

namespace CluE
{

/**
 * @brief Probability distribution on non-negative integers
 */
class FrequencyDistribution
{
public:
	/**
	* @brief constructs the centroid of the given vector of @ref FrequencyDistribution objects.
	*/
	FrequencyDistribution(std::vector<unsigned long> const& f);
	
	/**
	* @brief constructs optimal consolidated probability distribution
	*/
	FrequencyDistribution(std::vector<FrequencyDistribution*> const&);
	
	virtual ~FrequencyDistribution()
	{}
	
	virtual unsigned int size() const;
	
	virtual unsigned long frequency(unsigned int) const;
	
	virtual unsigned long total() const;

	virtual double probability(unsigned int) const;
	
	virtual double entropy() const;
	
	virtual double kullbackleibler(FrequencyDistribution const&) const;

private:
	std::vector<unsigned long> frequencies;
	std::vector<double> probabilities;
	unsigned long frequencysum;
};

std::ostream& operator << (std::ostream&, FrequencyDistribution&);

}

#endif
