#ifndef CARTERWEGMAN_H
#define CARTERWEGMAN_H

#include <limits>

#include "hashfunction.h"
#include "primegenerator.h"
#include "../misc/randomness.h"

namespace CluE
{

/**
 * @brief Universal hashing
 */
template<typename U, typename H> class CarterWegman : public HashFunction<U,H>
{
public:
	CarterWegman(U universeSize, H hashingSpaceSize = std::numeric_limits<H>::max());

	virtual H operator()(U const & element) const;

  /**
   * @brief Sets the hash value offset (default is 0)
   */
	void setOffset(H offset);
  
  /**
   * @brief Gets the hash value offset (default is 0)
   */
	H getOffset() const;

private:
	unsigned long lValue;
	unsigned long mValue;
	unsigned long nValue;
	unsigned long long pPrime;
	H hashingSpaceSize;
	H offset;
};

template<typename U, typename H> CarterWegman<U,H>::CarterWegman(U universeSize, H hashingSpaceSize) :
	hashingSpaceSize(hashingSpaceSize),
	offset(0)
{
	RandomGenerator gen = Randomness::getRandomGenerator();
	std::uniform_int_distribution<unsigned long long> dis(0, hashingSpaceSize-1);
	PrimeGenerator pg;

	lValue = dis(gen);
	mValue = dis(gen);
	nValue = dis(gen);
	pPrime = pg.getPrime(hashingSpaceSize, hashingSpaceSize*2, 0.00001);
}

template<typename U, typename H> H CarterWegman<U,H>::operator()(U const & element) const
{
	return H(((lValue * element * element + mValue * element + nValue) % pPrime) % hashingSpaceSize) + offset;
}

template<typename U, typename H> void CarterWegman<U,H>::setOffset(H offset)
{
	this->offset = offset;
}

template<typename U, typename H> H CarterWegman<U,H>::getOffset() const
{
	return offset;
}

}

#endif
