#ifndef PRIMEGENERATOR_H
#define PRIMEGENERATOR_H

#include <random>
#include <ctime>

#include "../misc/randomness.h"

namespace CluE
{

/**
 * @brief Generates numbers which are (most likely) prime numbers.
 */
class PrimeGenerator
{
//TODO Overflow check
//TODO Exceptions
//NOTE Performance tweaks to consider
// - More efficient sampling method

public:
	/**
	 * @brief Returns a number [lowerBound, upperBound] which is prime with probablity 1-errorProbability.
	 */
	unsigned long long getPrime(unsigned long long lowerBound, unsigned long long upperBound, double errorProbability);
	
	/**
	 * @brief Miller-Rabin primality test
	 */
	bool isMillerRabinPrime(unsigned long long candidate, double errorProbability);

private:	
	/**
	 * @brief Calculates "x^b mod m"
	 * 
	 * @sa Schneier, Bruce (1996). Applied Cryptography: Protocols, Algorithms, and Source Code in C, Second Edition (2nd ed.), p. 224. Wiley. ISBN 978-0471117094.
	 */
	unsigned long long potMod(unsigned long long x, unsigned long long b, unsigned long long m);
};

}

#endif