#ifndef AVERAGELINKAGE_H
#define AVERAGELINKAGE_H

#include "../base/dissimilaritymeasure.h"
#include "../base/clusterdissimilaritymeasure.h"

#include <vector>

namespace CluE
{

/**
 * @brief Average cluster linkage
 */
template<typename T> class AverageLinkage : public ClusterDissimilarityMeasure<T>
{
public:
	AverageLinkage(DissimilarityMeasure<T> const& measure);

	AverageLinkage(const AverageLinkage<T>&);
	AverageLinkage<T>& operator=(const AverageLinkage<T>&);
	virtual ~AverageLinkage();

	virtual AverageLinkage<T>* clone() const;

	/**
	 * @brief Computes the average linkage dissimilarity between the two given clusters.
	 *
	 * This function computes the dissimilarities between all (ordered) pairs of points from the
	 * two given clusters and returns the mean.
	 */
	virtual double dissimilarity(std::vector<T*> const& c1, std::vector<T*> const& c2);

private:
	DissimilarityMeasure<T>* measure;
};

template<typename T> AverageLinkage<T>::AverageLinkage(DissimilarityMeasure<T> const& m) :
	measure(m.clone())
{
}

template<typename T> AverageLinkage<T>::AverageLinkage(const AverageLinkage<T>& rhs) :
	ClusterDissimilarityMeasure<T>(rhs), measure(rhs.measure->clone())
{	
}

template<typename T> AverageLinkage<T>& AverageLinkage<T>::operator=(const AverageLinkage<T>& rhs)
{
	ClusterDissimilarityMeasure<T>::operator=(rhs);
	DissimilarityMeasure<T>* dm = rhs.measure->clone();
	delete this->measure;
	this->measure = dm;
	return *this;
}

template<typename T> AverageLinkage<T>::~AverageLinkage()
{
	delete this->measure;
}

template<typename T> AverageLinkage<T>* AverageLinkage<T>::clone() const
{
	return new AverageLinkage<T>(*this);
}

template<typename T> double AverageLinkage<T>::dissimilarity(std::vector<T*> const& c1,
	std::vector<T*> const& c2)
{
	double sum = 0;
	for (unsigned int i=0; i<c1.size(); i++)
		for (unsigned int j=0; j<c2.size(); j++)
		{
			sum+=this->measure->dissimilarity(*c1[i], *c2[j]);
			sum+=this->measure->dissimilarity(*c2[j], *c1[i]);
		}
	return sum/(c1.size()*c2.size());
}

}

#endif
