#ifndef COMPLETELINKAGE_H
#define COMPLETELINKAGE_H

#include "../base/dissimilaritymeasure.h"
#include "../base/clusterdissimilaritymeasure.h"

#include <vector>
#include <limits>

namespace CluE
{

/**
 * @brief Complete cluster linkage
 */
template<typename T> class CompleteLinkage : public ClusterDissimilarityMeasure<T>
{
public:
	CompleteLinkage(DissimilarityMeasure<T> const& measure);
	
	CompleteLinkage(const CompleteLinkage<T>&);
	CompleteLinkage<T>& operator=(const CompleteLinkage<T>&);
	virtual ~CompleteLinkage();

	virtual CompleteLinkage<T>* clone() const;

	/**
	 * @brief Computes the complete linkage dissimilarity between the two given clusters.
	 * 
	 * Computes the dissimilarities between all (ordered) pairs of points from the
	 * two given clusters and returns the maximum.
	*/
	virtual double dissimilarity(std::vector<T*> const& c1, std::vector<T*> const& c2);

private:
	DissimilarityMeasure<T>* measure;
};

template<typename T> CompleteLinkage<T>::CompleteLinkage(DissimilarityMeasure<T> const& m) :
	measure(m.clone())
{
}

template<typename T> CompleteLinkage<T>::CompleteLinkage(const CompleteLinkage<T>& rhs) :
	ClusterDissimilarityMeasure<T>(rhs), measure(rhs.measure->clone())
{	
}

template<typename T> CompleteLinkage<T>& CompleteLinkage<T>::operator=(const CompleteLinkage<T>& rhs)
{
	ClusterDissimilarityMeasure<T>::operator=(rhs);
	DissimilarityMeasure<T>* dm = rhs.measure->clone();
	delete this->measure;
	this->measure = dm;
	return *this;
}

template<typename T> CompleteLinkage<T>::~CompleteLinkage()
{
	delete this->measure;
}

template<typename T> CompleteLinkage<T>* CompleteLinkage<T>::clone() const
{
	return new CompleteLinkage<T>(*this);
}

template<typename T> double CompleteLinkage<T>::dissimilarity(std::vector<T*> const& c1,
	std::vector<T*> const& c2)
{
	double max = -std::numeric_limits<double>::infinity();
	for (unsigned int i=0; i<c1.size(); i++)
		for (unsigned int j=0; j<c2.size(); j++)
		{
			double cost = this->measure->dissimilarity(*c1[i], *c2[j]);
			if (cost > max)
				max = cost;
			cost = this->measure->dissimilarity(*c2[j], *c1[i]);
			if (cost > max)
				max = cost;
		}
	return max;
}

}

#endif