#ifndef POINTSETDISTANCE_H
#define POINTSETDISTANCE_H

#include "../base/dissimilaritymeasure.h"

namespace CluE
{

/**
 * @brief Calculates the (minimum) distance between a point and a set of points.
 * @remarks As this class is a comparision class for instant use, it does not create any copies of the provided objects / pointers (e.g. DissimilarityMeasure)
 * @ingroup helper_classes
 */
template<typename T> class PointSetDistance
{
public:
	PointSetDistance(DissimilarityMeasure<T> const & measure, std::set<T*> const & pointSet) :
		measure(measure),
		pointSet(pointSet)
	{
	}

	bool operator() (T const * const & obj1, T const * const & obj2);

private:
	DissimilarityMeasure<T> const & measure;
	std::set<T*> const & pointSet;
};

template<typename T> bool PointSetDistance<T>::operator() (T const * const & obj1, T const * const & obj2)
{
	if(pointSet.size() == 0)
		return true;
	
	double obj1min = measure.dissimilarity(*obj1, **pointSet.begin());
	double obj2min = measure.dissimilarity(*obj2, **pointSet.begin());
	for(typename std::set<T*>::iterator it = pointSet.begin()++; it != pointSet.end(); it++)
	{
		double candidate1 = measure.dissimilarity(*obj1, **it);
		if(candidate1 < obj1min)
			obj1min = candidate1;
		double candidate2 = measure.dissimilarity(*obj2, **it);
		if(candidate2 < obj2min)
			obj2min = candidate2;
	}
	
	return obj1min < obj2min;
}

}

#endif
