#ifndef SETITERATOR_H
#define SETITERATOR_H

#include <vector>
#include <set>

#include "../misc/flippinggraycodebits.h"

namespace CluE
{

/**
 * @brief Base class used to provide iterating over sets.
 */
template<class T> class SetIterator
{
public:
	/**
	 * @brief Size of the current element.
	 */
	virtual size_t size() const = 0;

	/**
	 * @brief Generates the next subset.
	 * @throw std::out_of_range No remaining elements.
	 */
	virtual void next() = 0;

	/**
	 * @brief Returns if there are more elements available.
	 */
	virtual bool hasMore() const = 0;

	/**
	 * @brief Returns the current element as a set.
	 */
	virtual std::set<T*> set() const = 0;

	/**
	 * @brief Returns the current element as a vector.
	 */
	virtual std::vector<T*> vector() const = 0;
};

}

#endif