#include <cmath>

#include "discreteboundedrealspaceprovider.h"
#include "../exception/invalidargumentexception.h"

namespace CluE
{


DiscreteBoundedRealSpaceProvider::DiscreteBoundedRealSpaceProvider(size_t dimension, unsigned long long n, Point minimum, double length) :
	_dimension(dimension),
	_n(n),
	minimum(minimum),
	stepSize()
{
	for(size_t i = 0; i < dimension; i++)
		stepSize.push_back(length / (n-1));
}
	
DiscreteBoundedRealSpaceProvider* DiscreteBoundedRealSpaceProvider::clone() const
{
	return new DiscreteBoundedRealSpaceProvider(*this);
}

Point DiscreteBoundedRealSpaceProvider::getVector(std::vector<unsigned long long> coordinates) const
{
	std::vector<double> coords;
	for(size_t i = 0; i < dimension(); i++)
	{
		unsigned long long c = coordinates[i];
		if(c < 0 || c >= n())
			throw InvalidArgumentException(0, "Invalid coordinate: exceeds bounds", "coordinates");
		coords.push_back(minimum[i] + stepSize[i] * c);
	}
	return Point(coords, 1.0);
}

std::vector<unsigned long long> DiscreteBoundedRealSpaceProvider::getCoordinates(Point const & vector) const
{
	std::vector<unsigned long long> coordinates;
	for(size_t i = 0; i < dimension(); i++)
	{
		double delta = vector[i] - minimum[i];
		unsigned long long steps = std::floor(delta / stepSize[i] + 0.5);
		if(steps < 0 || steps >= n())
			throw InvalidArgumentException(0, "Invalid point: exceeds bounds", "vector");
		coordinates.push_back(steps);
	}
	return coordinates;
}

}

