#ifndef DISCRETEBOUNDEDREALSPACEPROVIDER_H
#define DISCRETEBOUNDEDREALSPACEPROVIDER_H

#include "../point/point.h"
#include "../base/discreteboundedspace.h"

namespace CluE
{

/**
 * @brief Provides discrete bounded space features for Point.
 * 
 * @ingroup pointrelated_classes
 */
class DiscreteBoundedRealSpaceProvider : public DiscreteBoundedSpace<Point, unsigned long long>
{
public:
	DiscreteBoundedRealSpaceProvider(size_t dimension, unsigned long long n, Point minimum, double length);

	virtual DiscreteBoundedRealSpaceProvider* clone() const;
	
	virtual Point getVector(std::vector<unsigned long long> coordinates) const;
	
	virtual std::vector<unsigned long long> getCoordinates(Point const & vector) const;
	
	virtual size_t dimension() const
	{
		return _dimension;
	}
	
	virtual unsigned long long n() const
	{
		return _n;
	}
	
	virtual Point origin() const
	{
		return minimum;
	}
	
private:
	size_t _dimension;
	int _n;
	Point minimum;
	double length;
	std::vector<double> stepSize;
};

}

#endif
