#ifndef CLUEPOINTGAUSS_H
#define CLUEPOINTGAUSS_H

#include "../distribution/distribution.h"
#include "../point/point.h"
#include "../point/matrix.h"

#include <random>

#ifndef M_PI
#define M_PI           3.14159265358979323846
#endif

namespace CluE
{

/**
 * @brief Univariate normal distribution.
 *
 * @ingroup distribution_classes
 */
class PointGauss : public Distribution<Point>
{
public:

	PointGauss(Point const& m, Matrix const& cov);
	
	/**
	 * @brief Evaluates the density of the multivariate normal distribution at the given point x.
     */
	virtual double density(Point const& x) const;
	
	/**
	 * @brief Computes the negative log-likelihood of the density at the given point x.
     */
	virtual double nll(Point const& x) const;

	virtual double squaredMahalanobis(Point const& x) const;
	
	template<typename RndEngine> Point draw(RndEngine&) const;
	
private:
	Point mean;
	Matrix covariance, cholesky, inverseCholesky;
	double logSqrt; // log of sqrt of (2*pi)^d * det
};

template<typename RndEngine>
Point PointGauss::draw(RndEngine& re) const
{
	size_t d = this->mean.dimension();
	std::normal_distribution<> nd(0,1);

	std::vector<double> coord;
	for (size_t i=0; i<d; ++i)
		coord.push_back(nd(re));
	Matrix m = this->cholesky*Point(coord);

	coord.clear();
	for (size_t i=0; i<d; ++i)
		coord.push_back(m(i,0));
	return Point(coord)+this->mean;
}

}

#endif
