#include <math.h>

#include "../exception/invalidargumentexception.h"
#include "../point/pointgmm.h"
#include "../point/point.h"

using namespace CluE;

PointGMM::PointGMM(std::vector<double> const& w, std::vector<PointGauss> const& g)
	: weights(w), gaussians(g)
{
	if (w.size() != g.size())
		throw InvalidArgumentException(0, "Incompatible sizes!", "weights, gaussians");
}

/*PointGMM::PointGMM(GMMProvider const& p)
{
	this->weights = p.weights();
	size_t k = this->weights.size();
	for (size_t i=0; i<k; ++i)
		this->gaussians.push_back(PointGauss(p.mean(i), p.covariance(i)));
}
*/
double PointGMM::density(Point const& x) const
{
	size_t k = this->weights.size();
	double sum = 0;
	for (size_t i=0; i<k; ++i)
		sum += this->weights[i]*this->gaussians[i].density(x);
		
	return sum;
}

double PointGMM::nll(Point const& x) const
{
	return -log(this->density(x));
}

double PointGMM::minNLL(Point const& x) const
{
	size_t k = this->weights.size();
	double ret;
	if (k>0)
	{
		ret = this->gaussians[0].nll(x);
		for (size_t i=1; i<k; ++i)
		{
			double c = this->gaussians[i].nll(x);
			if (ret>c)
				ret = c;
		}
	}
	else
		ret = 0;
		
	return ret;
}

double PointGMM::minSquaredMahalanobis(Point const& x) const
{
	size_t k = this->weights.size();
	double ret;
	if (k>0)
	{
		ret = this->gaussians[0].squaredMahalanobis(x);
		for (size_t i=1; i<k; ++i)
		{
			double c = this->gaussians[i].squaredMahalanobis(x);
			if (ret>c)
				ret = c;
		}
	}
	else
		ret = 0;
		
	return ret;
}
