#ifndef CLUEPOINTGMM_H
#define CLUEPOINTGMM_H

#include "../exception/invalidruntimeconfigurationexception.h"
#include "../distribution/distribution.h"
#include "../point/pointgauss.h"
//#include "../learning/gmmprovider.h"
#include "../point/point.h"
#include "../point/matrix.h"

#include <random>

namespace CluE
{

/**
 * @brief Gaussian Mixture Model distribution.
 *
 * @ingroup distribution_classes
 */
class PointGMM : public Distribution<Point>
{
public:

	PointGMM(std::vector<double> const&, std::vector<PointGauss> const&);

//	PointGMM(GMMProvider const&);
	
	/**
	 * @brief Evaluates the density of the GMM distribution at the given point x.
     */
	virtual double density(Point const& x) const;
	
	/**
	 * @brief Computes the negative log-likelihood of the density at the given point x.
     */
	virtual double nll(Point const& x) const;
	
	virtual double minNLL(Point const& x) const;
	
	virtual double minSquaredMahalanobis(Point const& x) const;
	
	template<typename RndEngine> Point draw(RndEngine&) const;

private:
	std::vector<double> weights;
	std::vector<PointGauss> gaussians;
};

template<typename RndEngine>
Point PointGMM::draw(RndEngine& re) const
{
	size_t k = this->weights.size();
	if (k==0)
		throw InvalidRuntimeConfigurationException(0, "There are no gaussians in this mixture!");

	std::uniform_real_distribution<> urd(0, 1);
	double r = urd(re);
	
	size_t i = 0;
	while (r>this->weights[i])
	{
		++i;
		r-=this->weights[i];
	}
	if (i>=k)
		i=k-1;
		
	return this->gaussians[i].draw(re);
}

}

#endif
