#ifndef SQUAREDLPMETRIC_H
#define SQUAREDLPMETRIC_H

#include "../base/dissimilaritymeasure.h"
#include "../point/point.h"

namespace CluE
{

/**
 * @brief Squared Lp metric for Point
 * @remarks Use SquaredL1Metric / SquaredL2Metric instead of setting p=1 / p=2.
 *
 * @ingroup pointrelated_classes
 */
class SquaredLPMetric : public DissimilarityMeasure<Point>
{
public:
	SquaredLPMetric(double p);

	virtual SquaredLPMetric* clone() const;

	/**
	 * @brief Computes the squared Lp-distance between the two given Point instances.
	 */
	virtual double dissimilarity(Point const&, Point const&) const;

	/**
	 * @brief Sets p ("Squared Lp distance").
	 */
	void setP(double p);

private:
	double p;
};

}

#endif
