#ifndef DEMOPROCESSOR_H
#define DEMOPROCESSOR_H

#include <iostream>

#include <point/point.h>
#include <base/dissimilaritymeasure.h>
#include <point/pointcentroid.h>
#include <measure/singlelinkage.h>
#include <measure/averagelinkage.h>
#include <measure/completelinkage.h>

#include <datastructure/partitionsolution.h>
#include <datastructure/discreteproxysolution.h>
#include <datastructure/doublesolution.h>

#include <sampling/adaptivesampling.h>
#include <sampling/uniformsampling.h>

#include <clustering/agglomerative.h>
#include <clustering/lloydtype.h>
#include <clustering/gonzales.h>

#include <evaluation/maxdiamevaluator.h>
#include <evaluation/kmeansevaluator.h>

#include <misc/randomness.h>

namespace CluE
{
	class DemoProcessor
	{
	public:
		DemoProcessor(unsigned int dimension, unsigned int size, unsigned int clusters, bool useGauss);
		DemoProcessor(std::vector<Point*> input, unsigned int clusters);
		
		void agglomerativeSingleLinkage();
		void agglomerativeAverageLinkage();
		void agglomerativeCompleteLinkage();
		void lloydtypeKMeans(unsigned int iterations);
		void lloydtypeKMeansPP(unsigned int iterations);
		void gonzales();
		
		void setMeasure(DissimilarityMeasure<Point> *measure);
	private:
		void genGaussInput(unsigned int dimension, unsigned int size);
		void genUniformInput(unsigned int dimension, unsigned int size);
		std::vector<Point*> input;
		unsigned int clusters;
		DissimilarityMeasure<Point> *measure;
	};
}

#endif