#include <iostream>
#include <fstream>
#include <iomanip>
#include <climits>

#include "demoprocessor.h"

#include <point/point.h>
#include <base/dissimilaritymeasure.h>
#include <point/l1metric.h>
#include <point/squaredl1metric.h>
#include <point/l2metric.h>
#include <point/squaredl2metric.h>

using std::cin;
using std::cout;
using std::endl;

int argCount;
char** argValues;
int argPointer;

unsigned int getNumber(unsigned int maximum = INT_MAX, unsigned int minimum = 1) {
	unsigned int selection;
	if (argPointer < argCount) {
		selection = static_cast<unsigned int>(atoi(argValues[argPointer]));
		cout << selection << " (auto argument)" << endl;

		if(selection < minimum || selection > maximum) {
			cout << "Invalid auto argument!" << endl;
			exit(1);
		}
		argPointer++;
	} else {
		cin >> selection;

		while(selection < minimum || selection > maximum) {
			cout << selection << " is invalid." << endl;
			cin >> selection;
		}

	}

	return selection;
}

std::string getString() {
	std::string selection;
	if (argPointer < argCount) {
		selection = argValues[argPointer];
		cout << selection << " (auto argument)" << endl;

		argPointer++;
	} else {
		cin >> selection;
	}

	return selection;
}

int main(int argc, char **argv) {
	argCount = argc;
	argValues = argv;
	argPointer = 1;

	cout << "CluE demonstration" << endl << endl;

	std::clog.clear(std::ios_base::badbit);
	cout << std::setiosflags(std::ios::fixed) << std::setprecision(2);

	CluE::DemoProcessor *dp;
	unsigned int selection = 0;

	cout << "(1) Random input (Uniform)" << endl;
	cout << "(2) Random input (Gauss)" << endl;
	cout << "(3) From file" << endl;
	cout << "Select input: ";


	std::vector<CluE::Point*> points;
	selection = getNumber(3);
	switch(selection) {
	case 1:
	case 2: {
		cout << " Enter dimension: ";
		unsigned int dimension = getNumber();
		cout << " Enter number of points: ";
		unsigned int size = getNumber();
		cout << endl << "Enter desired number of clusters: ";
		unsigned int clusters = getNumber();

		dp = new CluE::DemoProcessor(dimension, size, clusters, selection == 2);
		break;
	}
	case 3: {
		cout << "One point per line, points formatted as \"x1,x2,x3,...,xn\"." << endl;
		cout << "Enter file path: ";
		std::string path = getString();

		std::ifstream filestream;
		filestream.open(path.c_str());
		std::string s;
		unsigned int num = 0;
		unsigned int dim = 0;
		while(getline(filestream, s)) {
			unsigned int tmpdim = 0;
			std::vector<double> coords;
			size_t pos = 0;
			size_t cut = s.find(",", pos);
			while(cut != std::string::npos) {
				coords.push_back(atof(s.substr(pos, cut-pos).c_str()));
				pos = cut+1;
				cut = s.find(",", pos);
				tmpdim++;
			}
			coords.push_back(atof(s.substr(pos).c_str()));

			if(dim == 0 || dim == tmpdim) {
				dim = tmpdim;
				CluE::Point *p = new CluE::Point(coords, 1);
				points.push_back(p);
				num++;
			}
		}
		filestream.close();
		cout << num << " points read." << endl;

		if(num == 0) {
			cout << endl << "Aborted." << endl;
			return 1;
		}

		cout << endl << "Enter desired number of clusters: ";
		unsigned int clusters = getNumber();

		dp = new CluE::DemoProcessor(points, clusters);
		break;
	}
	}

	cout << "(1) L1 distance" << endl;
	cout << "(2) Squared L1 distance" << endl;
	cout << "(3) L2 distance" << endl;
	cout << "(4) Squared L2 distance" << endl;

	cout << "Select measure: ";
	selection = getNumber(4);
	cout << endl;

	CluE::DissimilarityMeasure<CluE::Point> *m;
	switch(selection) {
	case 1:
		m = new CluE::L1Metric();
		break;
	case 2:
		m = new CluE::SquaredL1Metric();
		break;
	case 3:
		m = new CluE::L2Metric();
		break;
	case 4:
		m = new CluE::SquaredL2Metric();
		break;
	}
	dp->setMeasure(m);

	cout << "(1) Agglomerative (single linkage)" << endl;
	cout << "(2) Agglomerative (average linkage)" << endl;
	cout << "(3) Agglomerative (complete linkage)" << endl;
	cout << "(4) K-means" << endl;
	cout << "(5) K-means++" << endl;
	cout << "(6) Gonzales" << endl;
	cout << "(0) Exit" << endl;
	bool ex = false;
	while(!ex) {
		cout << "Select clustering algorithm: ";

		selection = getNumber(6, 0);
		cout << endl;

		switch(selection) {
		case 0:
			ex = true;
			break;
		case 1:
			dp->agglomerativeSingleLinkage();
			break;
		case 2:
			dp->agglomerativeAverageLinkage();
			break;
		case 3:
			dp->agglomerativeCompleteLinkage();
			break;
		case 4:
			dp->lloydtypeKMeans(0);
			break;
		case 5:
			dp->lloydtypeKMeansPP(0);
			break;
		case 6:
			dp->gonzales();
			break;
		}
	}

	delete dp;
	delete m;
	size_t size = points.size();
	for(size_t i = 0; i < size; i++)
		delete points[i];

	cout << "Finished." << endl;

	return 0;
}
