#ifndef ALGORITHMTEST_H
#define ALGORITHMTEST_H

#include <string>
#include <vector>
#include <unordered_map>

#include "enumerations.h"
#include "filereader.h"

#include <point/point.h>
#include <base/solutionprovider.h>
#include <base/dissimilaritymeasure.h>
#include <point/pointcentroid.h>

namespace CluE {

class AlgorithmTest {
public:
	//AlgorithmTest_main.cpp
	AlgorithmTest(std::string path, std::ostream& stream);
	~AlgorithmTest();

	void setupDiscreteProxySequenceTest(std::vector<unsigned int> proxysequence);
	void setupPartitioningTest(std::vector<std::vector<unsigned int> > partitioning);
	void setupEpsilonApproximationTest(unsigned int laps, double optimalCost, double propability, double epsilon);
	bool run();

private:
	FileReader configuration;

	AlgorithmType algorithm;
	std::vector<Point*> input;
	SolutionProvider* result;
	DissimilarityMeasure<Point> *measure;

	unsigned int solution;
	std::unordered_map<Point*, unsigned int> lookupIndex;
	std::ostream& output;

	//AlgorithmTest_main.cpp
	void autoSetup();
	void runAlgorithm(bool printOutput);

	//AlgorithmTest_dps.cpp
	bool testDiscreteProxySequence;
	std::vector<Point*> discreteProxySequence;
	bool doDiscreteProxySequenceTest();

	//AlgorithmTest_part.cpp
	bool testPartitioning;
	std::vector<std::vector<Point*> > partitions;
	bool doPartitioningTest();

	//AlgorithmTest_epsilonapp.cpp
	bool testEpsilonApproximation;
	unsigned int epsilonAppLaps;
	double epsilonAppOptimalCost;
	double epsilonAppPropability;
	double epsilonAppEpsilon;
	bool doEpsilonApproximationTest();

	//AlgorithmTest_helper.cpp
	void printSuccess(bool success);
	
	//Algorithm helpers
	double lastCost;
	PointCentroid* centroidGen;
};
}

#endif
