#include <stdexcept>

#include "algorithmtest.h"
#include "exceptions.h"

#include <base/discreteproxyprovider.h>

using std::string;
using std::endl;
using std::vector;

namespace CluE {
	
void AlgorithmTest::setupDiscreteProxySequenceTest(std::vector<unsigned int> proxysequence) {
	try {
		size_t n = proxysequence.size();
		for(size_t i = 0; i < n; i++)
			discreteProxySequence.push_back(input.at(proxysequence[i]));
	} catch(std::out_of_range ex) {
		throw NoSuchPoint();
	}
}

bool AlgorithmTest::doDiscreteProxySequenceTest() {
	DiscreteProxyProvider<Point> *dpp = dynamic_cast<DiscreteProxyProvider<Point>*>(result);
	if(dpp == 0) {
		InvalidConfiguration ex;
		ex.message = "Algorithm does not provide this functionality.";
		throw ex;
	}
	
	std::vector<Point*> proxies = dpp->discrete_proxies(solution);
	size_t numProxies = proxies.size();
	if(numProxies != discreteProxySequence.size()) {
		output << "Number of expected proxies != number of returned proxies" << endl;
		return false;
	}

	bool result = true;

	for(size_t i = 0; i < numProxies; i++) {
		bool tmp = proxies[i] == discreteProxySequence[i];
		if(!tmp) {
			output << "Proxy " << i+1 << " error: Expected " << lookupIndex.at(discreteProxySequence[i]) << ", returned " << lookupIndex.at(proxies[i]) << " instead." << endl;
		}
		result &= tmp;
	}

	return result;
}

}
