#include <sstream>
#include <stdexcept>

#include "helper.h"
#include "algorithmtest.h"
#include "exceptions.h"

#include <base/partitionprovider.h>

using std::string;
using std::endl;
using std::vector;

namespace CluE {

void AlgorithmTest::setupPartitioningTest(std::vector<std::vector<unsigned int> > partitioning) {
	try {
		size_t n = partitioning.size();
		for(size_t i = 0; i < n; i++) {
			vector<Point*> partition;
			size_t m = partitioning[i].size();
			for(size_t j = 0; j < m; j++) {
				partition.push_back(input.at(partitioning[i][j]));
			}
			partitions.push_back(partition);
		}
	} catch(std::out_of_range ex) {
		throw NoSuchPoint();
	}
}

bool AlgorithmTest::doPartitioningTest() {
	PartitionProvider<Point> *ps = dynamic_cast<PartitionProvider<Point>*>(result);
	if(ps == 0) {
		InvalidConfiguration ex;
		ex.message = "Algorithm does not provide this functionality.";
		throw ex;
	}

	vector<vector<Point*> > sol = ps->clustering(solution);

	size_t returnedNumber = 0;
	size_t solutionSize = sol.size();
	std::unordered_map<Point*, size_t> ltb;
	{
		size_t n = solutionSize;
		for(size_t i = 0; i < n; i++) {
			size_t m = sol[i].size();
			for(size_t j = 0; j < m; j++) {
				ltb[sol[i][j]] = i;
			}
			returnedNumber += m;
		}
	}

	bool suc = true;
	unsigned int expectedSize = configuration.getConfigUInt("partitioning.number");
	size_t expectedNumber = 0;
	{
		unsigned int n = expectedSize;
		for(unsigned int i = 0; i < n; i++) {
			std::stringstream key;
			key << "partitioning.partition" << i;
			string val = configuration.getConfigString(key.str());
			vector<unsigned int> partition = Helper::convertToUIntSequence(val);

			unsigned int expectedIn = ltb.at(input[partition[0]]);
			unsigned int m = partition.size();
			for(unsigned int j = 1; j < m; j++) {
				if(ltb.at(input[partition[j]]) != expectedIn) {
					output << "Point " << partition[j] << " missing in partition " << i << "." << endl;
					suc = false;
				}
			}
			expectedNumber += m;
		}
	}

	if(solutionSize != expectedSize) {
		output << "Expected " << expectedSize << " partitions but got " << solutionSize << "." << endl;
		suc = false;
	} else if(returnedNumber != expectedNumber) {
		output << "Expected " << expectedNumber << " points but got " << returnedNumber << "." << endl;
		suc = false;
	}

	return suc;
}

}
