#ifndef EVALUATIONTEST_H
#define EVALUATIONTEST_H

#include <string>
#include <vector>
#include <unordered_map>

#include "enumerations.h"
#include "filereader.h"

#include <point/point.h>
#include <base/evaluation.h>
#include <base/dissimilaritymeasure.h>

#include <base/proxyevaluation.h>

namespace CluE {

class EvaluationTest {
public:
	//EvaluationTest_main.cpp
	EvaluationTest(std::string path, std::ostream& stream);
	~EvaluationTest();

	void setupProxyEvaluationTest(std::vector<unsigned int> proxyProxies, std::vector<double> proxyProxycosts);
	void setupPartitionEvaluationTest(std::vector<std::vector<unsigned int> > partPartitions, std::vector<double> partPartitioncosts);
	void setupCombinedEvaluationTest(std::vector<std::vector<unsigned int> > partitioning, std::vector<double> partitioningcosts, std::vector<unsigned int> proxyProxies);

	bool run();

private:
	FileReader configuration;

	EvaluationType algorithm;
	Evaluation *evaluation;
	std::vector<Point*> input;
	DissimilarityMeasure<Point> *measure;

	unsigned int solution;
	std::unordered_map<Point*, unsigned int> lookupIndex;
	std::ostream& output;

	//EvaluationTest_proxy.cpp
	bool testProxyEvaluation;
	std::vector<Point*> proxyProxies;
	std::vector<double> proxyProxycosts;
	bool doProxyEvaluationTest();

	//EvaluationTest_part.cpp
	bool testPartitionEvaluation;
	std::vector<std::vector<Point*> > partPartitions;
	std::vector<double> partPartitioncosts;
	bool doPartitionEvaluationTest();

	//EvaluationTest_combined.cpp
	bool testCombinedEvaluation;
	std::vector<Point*> combinedProxies;
	std::vector<std::vector<Point*> > combinedPartitions;
	std::vector<double> combinedPartitioncosts;
	bool doCombinedEvaluationTest();

	//EvaluationTest_main.cpp
	void autoSetup();

	//EvaluationTest_helper.cpp
	void printSuccess(bool success);
	static std::vector<unsigned int> convertToUIntSequence(std::string val);
};
}

#endif
