#include <stdexcept>

#include "helper.h"
#include "exceptions.h"
#include "evaluationtest.h"

#include <base/combinedevaluation.h>

using std::string;
using std::endl;
using std::vector;

namespace CluE {

void EvaluationTest::setupCombinedEvaluationTest(std::vector<std::vector<unsigned int> > partitioning, std::vector<double> partitioningcosts, std::vector<unsigned int> proxies) {
	try {
		{
			size_t n = proxies.size();
			for(size_t i = 0; i < n; i++) {
				combinedProxies.push_back(input.at(proxies[i]));
			}

		}
		{
			size_t n = partitioning.size();
			if(n != partitioningcosts.size()) {
				MismatchingSizes ex;
				ex.message = "Number of proxies and number of given proxycosts are not equal.";
				throw ex;
			}

			for(size_t i = 0; i < n; i++) {
				vector<Point*> partition;
				size_t m = partitioning[i].size();
				for(size_t j = 0; j < m; j++) {
					partition.push_back(input.at(partitioning[i][j]));
				}
				combinedPartitions.push_back(partition);
			}
			combinedPartitioncosts = partitioningcosts;
		}
	} catch(std::out_of_range ex) {
		throw NoSuchPoint();
	}
};

bool EvaluationTest::doCombinedEvaluationTest() {
	bool success = true;

	CombinedEvaluation<Point> *eval = dynamic_cast<CombinedEvaluation<Point>*>(evaluation);
	if(eval == 0) {
		InvalidConfiguration ex;
		ex.message = "Algorithm does not provide this functionality.";
		throw ex;
	}

	size_t numPartition = combinedPartitions.size();
	for(size_t i = 0; i < numPartition; i++) {
		double cost = eval->combinedcost(combinedPartitions[i], combinedProxies[i]);
		if(!Helper::equalsDouble(cost, combinedPartitioncosts[i])) {
			success= false;
			output << "Partition " << i+1 << " error: Expected cost " << combinedPartitioncosts[i] << ", returned " << cost << " instead." << std::endl;
		}
	}

	return success;
}

}
