#include <stdexcept>

#include "helper.h"
#include "exceptions.h"
#include "evaluationtest.h"

#include <base/partitionevaluation.h>

using std::string;
using std::endl;
using std::vector;

namespace CluE {

void EvaluationTest::setupPartitionEvaluationTest(std::vector<std::vector<unsigned int> > partitioning, std::vector<double> partitioningcosts) {
	try {
		size_t n = partitioning.size();
		if(n != partitioningcosts.size()) {
			MismatchingSizes ex;
			ex.message = "Number of proxies and number of given proxycosts are not equal.";
			throw ex;
		}

		for(size_t i = 0; i < n; i++) {
			vector<Point*> partition;
			size_t m = partitioning[i].size();
			for(size_t j = 0; j < m; j++) {
				partition.push_back(input.at(partitioning[i][j]));
			}
			partPartitions.push_back(partition);
		}
		partPartitioncosts = partitioningcosts;
	} catch(std::out_of_range ex) {
		throw NoSuchPoint();
	}
};

bool EvaluationTest::doPartitionEvaluationTest() {
	bool success = true;

	PartitionEvaluation<Point> *eval = dynamic_cast<PartitionEvaluation<Point>*>(evaluation);
	if(eval == 0) {
		InvalidConfiguration ex;
		ex.message = "Algorithm does not provide this functionality.";
		throw ex;
	}

	size_t numPartition = partPartitions.size();
	for(size_t i = 0; i < numPartition; i++) {
		double cost = eval->partitioncost(partPartitions[i]);
		if(!Helper::equalsDouble(cost, partPartitioncosts[i])) {
			success= false;
			output << "Partition " << i+1 << " error: Expected cost " << partPartitioncosts[i] << ", returned " << cost << " instead." << std::endl;
		}
	}

	return success;
}

}
