#include <stdexcept>

#include "helper.h"
#include "exceptions.h"
#include "evaluationtest.h"

#include <base/proxyevaluation.h>

namespace CluE {

void EvaluationTest::setupProxyEvaluationTest(std::vector<unsigned int> p, std::vector<double> pc) {
	try {
		size_t n = p.size();
		if(n != pc.size()) {
			MismatchingSizes ex;
			ex.message = "Number of proxies and number of given proxycosts are not equal.";
			throw ex;
		}
		for(size_t i = 0; i < n; i++) {
			proxyProxies.push_back(input.at(p[i]));
		}
		proxyProxycosts = pc;
	} catch(std::out_of_range ex) {
		throw NoSuchPoint();
	}
};

bool EvaluationTest::doProxyEvaluationTest() {
	bool success = true;

	ProxyEvaluation<Point> *eval = dynamic_cast<ProxyEvaluation<Point>*>(evaluation);
	if(eval == 0) {
		InvalidConfiguration ex;
		ex.message = "Algorithm does not provide this functionality.";
		throw ex;
	}

	size_t numProxies = proxyProxies.size();
	for(size_t i = 0; i < numProxies; i++) {
		double cost = eval->proxycost(input, proxyProxies, i);
		if(!Helper::equalsDouble(cost, proxyProxycosts[i]))
		{
			success= false;
			output << "Proxy " << i+1 << " error: Expected cost " << proxyProxycosts[i] << ", returned " << cost << " instead." << std::endl;
		}
	}

	return success;
}

}
