#include "exceptions.h"
#include "filereader.h"

#include <cstdlib>
#include <fstream>
#include <boost/algorithm/string.hpp>

using std::string;

namespace CluE
{

FileReader::FileReader(std::string path)
{
	std::ifstream filestream;
	filestream.open(path.c_str());
	std::string line;
	string prefix = "";
	while(getline(filestream, line))
	{
		int section = line.substr(0,1).compare("[");
		if(section == 0)
		{
			prefix = line.substr(1, line.length()-2) + ".";
			continue;
		}

		size_t cut = line.find("=");
		if(cut != string::npos)
		{
			string key = prefix + line.substr(0, cut);
			string value = line.substr(cut+1);
			boost::to_lower(key);
			boost::trim(key);
			boost::trim(value);
			keyvalue.insert(std::pair<string,string> (key, value));
		}

	}
	filestream.close();
}

bool FileReader::hasValue(std::string key)
{
    boost::to_lower(key);
	std::map<string, string>::iterator element = keyvalue.find(key);
    return element != keyvalue.end();
}

string FileReader::getValue(string key, bool toLower)
{
    boost::to_lower(key);
	std::map<string, string>::iterator element = keyvalue.find(key);
	if(element != keyvalue.end())
	{
		string ret = element->second;
		if(toLower)
			boost::to_lower(ret);
	    return ret;
	}
	else
	{
	    return "";
	}
}

std::vector<CluE::Point*> FileReader::readPointFile(std::string path)
{
	std::vector<CluE::Point*> points;

	std::ifstream filestream;
	filestream.open(path.c_str());
	std::string s;
	unsigned int num = 0;
	while(getline(filestream, s))
	{
		std::vector<double> coords;
		size_t pos = 0;
		size_t cut = s.find(",", pos);
		while(cut != std::string::npos)
		{
			coords.push_back(std::atof(s.substr(pos, cut-pos).c_str()));
			pos = cut+1;
			cut = s.find(",", pos);
		}
		coords.push_back(atof(s.substr(pos).c_str()));

		CluE::Point *p = new CluE::Point(coords, 1);
		points.push_back(p);
		num++;
	}
	filestream.close();
	
	if(num == 0)
	{
		InvalidFile ex;
		ex.message = "Empty input file.";
		throw ex;
	}

	return points;
}

bool FileReader::getConfigFlag(string key) {
	if(hasValue(key)) {
		string value = getValue(key);
		if(value.compare("false") == 0)
			return false;
		else
			return true;
	} else {
		return false;
	}
}

std::string FileReader::getConfigString(std::string key, bool toLower) {
	if(hasValue(key)) {
		return getValue(key, toLower);
	} else {
		InvalidConfiguration ic;
		ic.message = key + ": Missing.";
		throw ic;
	}
}

unsigned int FileReader::getConfigUInt(string key) {
	string value = getConfigString(key);
	unsigned int ret = static_cast<unsigned int>(std::atoi(value.c_str()));
	return ret;
}

double FileReader::getConfigDouble(std::string key){
	string value = getConfigString(key);
	double ret = static_cast<double>(std::atof(value.c_str()));
	return ret;
}

}
