#include <iostream>
#include <iomanip>

#include "filereader.h"
#include "algorithmtest.h"
#include "evaluationtest.h"

using std::endl;
using namespace CluE;

int main(int argc, char **argv)
{
	if(argc < 2)
	{
		std::cout << "No argument(s) / test case files." << std::endl;
		return 1;
	}

	//Suppress CluE messages
	std::clog.clear(std::ios_base::badbit);
	std::cout << std::setiosflags(std::ios::fixed) << std::setprecision(2);

	std::ostream& out = std::cout;

	bool success = true;
	for(int i = 1; i < argc; i++)
	{
		std::string testfile = argv[i];

		FileReader fr(testfile);
		if(fr.hasValue("main.testtype"))
		{
			std::string m = fr.getConfigString("main.testtype");
			if(m.compare("algorithm") == 0)
			{
				out << "### ALGORITHM TEST: " << testfile << " ###" << endl;
				AlgorithmTest tc(testfile, out);
				success &= tc.run();
			}
			else if(m.compare("evaluation") == 0)
			{
				out << "### EVALUATION TEST: " << testfile << " ###" << endl;
				EvaluationTest tc(testfile, out);
				success &= tc.run();
			}
		}

	}

	if(success)
	{
		out << endl << "Finished successfully." << endl;
		return 0;
	}
	else
	{
		out << endl << "FINISHED WITH ERRORS!" << endl;
		return 1;
	}
}
